#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import os

import roomie_bot.expenses.expenses as expenses
from roomie_bot.database.database import Database

from telegram.ext import Updater, CommandHandler


# Enable logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    level=logging.INFO)

logger = logging.getLogger(__name__)


# Define a few command handlers.
def start(update, context):
    update.message.reply_text(
        '''
        Hi! I am roomie, a bot with tools to make life easier between roommates.
        \nI can help you tracking your expenses or organizing tasks
        \nFirst, every user should /register (you should have a username set up)
        \n\nSend /help for a list of commands
        '''
    )


def register(update, context):
    if update.effective_user.username is None:
        update.message.reply_text('You need an username to /register'
                                  '\n\nTo set up one go to settings in'
                                  ' telegram and select "username"')
    else:
        db = Database()
        db.register_user(update.effective_user.id, update.effective_user.username)
        db.close()

        update.message.reply_text('Successfully registered with username ' + update.effective_user.username)


def pay(update, context):
    errors = ''

    # Check argument size and format
    if len(context.args) >= 2:
        # Check correct format for money
        if expenses.is_number(context.args[0]):
            money = float(context.args[0])
        else:
            errors += 'Error: {} is not an amount of money\n'.format(context.args[0])

        db = Database()
        # Check payer is registered
        if db.get_username(update.effective_user.id) is None:
            errors += 'Error: You are not registered\n'

        # Check debtors are registered
        debtors_id = []

        for debtor in context.args[1:]:
            userid = db.get_userid(debtor[1:])
            if userid is None:
                errors += 'Error: {} is not registered\n'.format(debtor)
            else:
                debtors_id.append(userid)
        db.close()
    else:
        errors += 'Format: /pay money @debtor1 [@debtor2..]'

    if errors:
        if errors.split()[0] != 'Format:':
            errors += 'Format: /pay money @debtor1 [@debtor2..]'
        update.message.reply_text(errors)
    else:
        expenses.new_payment(update.effective_user.id, update.message.chat_id,
                             money, debtors_id, context.args[1:])

        update.message.reply_text('Payment added succesfully'
                                  '\nCheck new debts with /debts')


def history(update, context):
    history = expenses.get_payments(update.effective_chat.id)

    if history == '':
        update.message.reply_text("No history yet")
    else:
        update.message.reply_text(history)


def debts(update, context):
    debts = expenses.get_debts(update.effective_chat.id)

    if debts == "":
        update.message.reply_text("No debts yet")
    else:
        update.message.reply_text(debts)


def help(update, context):
    update.message.reply_text('Help!')


def error(update, context):
    """Log Errors caused by Updates."""
    logger.warning('Update "%s" caused error "%s"', update, context.error)


def main():
    # Initialize database
    db = Database()
    db.setup()
    db.close()

    # Create the Updater and pass it bot's token.
    TOKEN = os.environ.get("TOKEN")
    updater = Updater(TOKEN, use_context=True)

    # Get the dispatcher to register handlers
    dp = updater.dispatcher

    # on different commands - answer in Telegram
    dp.add_handler(CommandHandler("start", start))
    dp.add_handler(CommandHandler("register", register))
    dp.add_handler(CommandHandler("pay", pay))
    dp.add_handler(CommandHandler("history", history))
    dp.add_handler(CommandHandler("debts", debts))
    dp.add_handler(CommandHandler("help", help))

    # log all errors
    dp.add_error_handler(error)

    # Start the Bot
    updater.start_polling()

    # Run the bot until interrupted
    updater.idle()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from roomie_bot.database.database import Database

DBNAME = "room.sqlite"


def is_number(text: str):
    try:
        float(text)
        return True
    except ValueError:
        return False


def get_debts(chat_id: int, dbname: str = DBNAME):
    reply = ''
    db = Database(dbname)

    debts = db.get_debts(chat_id)
    for debt in debts:
        reply += '{}: {}€\n'.format(db.get_username(debt[0]), debt[1])

    db.close()

    return reply


def get_payments(chat_id: int, dbname: str = DBNAME):
    reply = ''
    db = Database(dbname)

    payments = db.get_payments(chat_id)
    for payment in payments:
        reply += '{} payed {} for {}\n'.format(db.get_username(payment[0]), payment[1], payment[2])

    db.close()

    return reply


def new_payment(payer_id: int, chat_id: int, money: float, debtors_id, debtors_username, round_dec: int = 2, dbname: str = DBNAME):
    db = Database(dbname)

    # Update payer debt
    debt = db.get_debt(payer_id, chat_id)

    if debt is None:
        db.add_debt(payer_id, chat_id, round(money, round_dec))
    else:
        db.update_debt(payer_id, chat_id, round(debt + money, round_dec))

    # Update debtors debt
    extra_debt = money / len(debtors_id)

    for user_id in debtors_id:
        debt = db.get_debt(user_id, chat_id)

        if debt is None:
            db.add_debt(user_id, chat_id, round(-extra_debt, round_dec))
        else:
            db.update_debt(user_id, chat_id, round(debt - extra_debt, round_dec))

    # Add payment to history
    desc = ' '

    db.add_payment(payer_id, chat_id, money, desc.join(debtors_username))

    db.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sqlite3


class Database:
    def __init__(self, dbname="room.sqlite"):
        self._dbname = dbname
        self._conn = sqlite3.connect(dbname)
        self._c = self._conn.cursor()

        self._c.execute('PRAGMA foreign_keys = ON')

    def setup(self):
        self._c.execute('''
                        CREATE TABLE IF NOT EXISTS users (
                            user_id INT PRIMARY KEY,
                            username TEXT NOT NULL UNIQUE
                        );
                        ''')
        self._c.execute('''
                        CREATE TABLE IF NOT EXISTS debts (
                            user_id INT,
                            chat_id INT,
                            debt DOUBLE NOT NULL DEFAULT 0,
                            PRIMARY KEY (user_id, chat_id),
                            FOREIGN KEY (user_id) REFERENCES users(user_id)
                            ON DELETE CASCADE
                        );
                        ''')
        self._c.execute('''
                        CREATE TABLE IF NOT EXISTS payments (
                            payment_id INTEGER PRIMARY KEY,
                            user_id INT,
                            chat_id INT,
                            money DOUBLE,
                            description TEXT,
                            FOREIGN KEY (user_id, chat_id)
                            REFERENCES debts(user_id, chat_id)
                            ON DELETE CASCADE
                            ON UPDATE RESTRICT
                        );
                        ''')
        self._conn.commit()

    def clean(self):
        self._c.execute('DROP TABLE payments')
        self._c.execute('DROP TABLE debts')
        self._c.execute('DROP TABLE users')

    def close(self):
        self._conn.close()

    # Register user
    def register_user(self, user_id: int, username: str):
        self._c.execute('''
                        INSERT OR REPLACE INTO users (user_id, username)
                        VALUES (?, ?);
                        ''', (user_id, username, ))
        self._conn.commit()

    def get_username(self, user_id: int):
        user = self._c.execute('SELECT username FROM users WHERE user_id = ?',
                               (user_id, )).fetchone()

        if user is not None:
            return user[0]

    def get_userid(self, username: str):
        user = self._c.execute('SELECT user_id FROM users WHERE username = ?',
                               (username, )).fetchone()

        if user is not None:
            return user[0]

    def remove_user(self, user_id: int):
        self._c.execute('DELETE FROM users WHERE user_id = ?', (user_id, ))
        self._conn.commit()

    # Expenses related
    def add_payment(self, user_id: int, chat_id: int, money: float, description: str):
        self._c.execute('''
                        INSERT INTO payments (user_id, chat_id, money, description)
                        VALUES (?, ?, ?, ?);
                        ''', (user_id, chat_id, money, description, ))
        self._conn.commit()

    def get_payment(self, payment_id: int):
        payment = self._c.execute('''
                                  SELECT user_id, chat_id, money, description
                                  FROM payments WHERE payment_id = ?
                                  ''', (payment_id, )).fetchone()

        if payment is not None:
            return payment

    def get_payments(self, chat_id: int, n: int = 10):
        return self._c.execute('''
                                SELECT user_id, money, description FROM payments
                                WHERE chat_id = ?
                                ORDER BY payment_id DESC
                                LIMIT ?
                                ''', (chat_id, n, )).fetchall()

    def add_debt(self, user_id: int, chat_id: int, debt: float):
        self._c.execute('INSERT INTO debts (user_id, chat_id, debt) VALUES (?, ?, ?);', (user_id, chat_id, debt, ))
        self._conn.commit()

    def update_debt(self, user_id: int, chat_id: int, debt: float):
        self._c.execute('''
                        UPDATE debts
                        SET debt = ?
                        WHERE user_id = ? AND chat_id = ?;
                        ''', (debt, user_id, chat_id, ))
        self._conn.commit()

    def get_debt(self, user_id: int, chat_id: int):
        debt = self._c.execute('SELECT debt FROM debts WHERE user_id = ? AND chat_id = ?',
                               (user_id, chat_id, )).fetchone()

        if debt is not None:
            return debt[0]

    def get_debts(self, chat_id: int):
        return self._c.execute('SELECT user_id, debt FROM debts WHERE chat_id = ?', (chat_id, )).fetchall()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['roomie_bot',
 'roomie_bot.api_rest',
 'roomie_bot.database',
 'roomie_bot.expenses']

package_data = \
{'': ['*']}

install_requires = \
['flask>=1.1.2,<2.0.0', 'python-telegram-bot>=12.8,<13.0']

setup_kwargs = {
    'name': 'roomie-bot',
    'version': '0.1.0',
    'description': 'Telegram bot with utilities for roommates',
    'long_description': None,
    'author': 'Dipzza',
    'author_email': 'dipzza@protonmail.ch',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from flask import Flask, jsonify

from roomie_bot.database.database import Database

app = Flask(__name__)
app.config["DATABASE"] = "room.sqlite"


@app.route('/api/users/<int:user_id>')
def get_user(user_id):
    db = Database(app.config["DATABASE"])
    username = db.get_username(user_id)
    db.close()

    if username is None:
        return jsonify({})
    else:
        return jsonify({'user_id': user_id,
                        'username': username})


@app.route('/api/debts/<int:user_id>/<int:chat_id>')
def get_debt(user_id, chat_id):
    db = Database(app.config["DATABASE"])
    debt = db.get_debt(user_id, chat_id)
    db.close()

    if debt is None:
        return jsonify({})
    else:
        return jsonify({'user_id': user_id,
                        'chat_id': chat_id,
                        'debt': debt})


@app.route('/api/payments/<int:payment_id>')
def get_payment(payment_id):
    db = Database(app.config["DATABASE"])
    payment = db.get_payment(payment_id)
    db.close()

    if payment is None:
        return jsonify({})
    else:
        return jsonify({'user_id': payment[0],
                        'chat_id': payment[1],
                        'money': payment[2],
                        'desc': payment[3]})
